// Source          ::     RoverV2_WiFi_V1 ( Merch 2015 )

// Description     ::     Basic Rover WASD(EX) control program using WiFi
//
//                        This program will also use an ultrasonic sensor, if installed and code activated, to prevent it from running into obstacles.

//                        WiFi access would typically be done through a local router,
//                        but by using Dynamic DNS ie http://www.noip.com/remote-access you could control the Rover
//                        from anywhere in the world, although flying blind would be challenging.
//
//                        Available commands ...
//
//                                   ( W )     - Forward
//                                   ( A )     - Left
//                                   ( S )     - Back
//                                   ( D )     - Right
//                                   ( E )     - Stop
//                                   ( X )     - Reboot
//
//                                   ( - )     - Decrease left  motor speed by 10%
//                                   ( = )     - Decrease right motor speed by 10%
//
//                                   ( _ )     - Increase left  motor speed by 10%
//                                   ( + )     - Increase right motor speed by 10%
//
//                                   ( 1 thru 9 ) - Speed Control, both motors, in %
//                                                 ( 4 == 90% , 3 == 80% , 2 == 70% , 1 == 60% ... of BASELINESPEED )
//                                                 ( 5 is BASELINESPEED )
//                                                 ( 6 == 110% , 7 == 120% , 8 == 130% , 9 == 140% )



//                        Recommended components ( or equivalent ) 

//                        1)   WiFi Router ... it is assumed you have a basic understanding of networking.
//                             Basically, the Rover will connect to the router to receive an IP address making it a
//                             simple computer on a network, and then the pilot will connect to the same network using...

//                        2)   A Terminal Emulator such as PuTTY.
//                             The pilot will use this to connect to the IP address assigned to the Rover,
//                             and then send commands and receive confirmation messages.

//                        3)   http://www.robotshop.com/ca/en/dfrobotshop-rover-tracked-robot-basic-kit.html
//                        4)   http://www.robotshop.com/ca/en/arduino-wifi-shield-integrated-antenna.html
//                        5)   http://www.robotshop.com/ca/en/dfrobot-7-4v-lipo-2200mah-battery.html
//                        6)   http://www.robotshop.com/ca/en/seeedstudio-ultrasonic-range-finder.html


// Original Program   ::     Rover control & program standards     - Joe M                        / arduino (at) bellaliant.net   / March 2015
//                           WiFi Connectivity                     - Rezk Bouras aka (Prudent++)  / Student, NBCC Saint John      / March 2015
//                          

// Modified By          ::   Your Name Here / Contact / Date  



//  This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License 
//  as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  Lesser General Public License for more details.
//
//  For a copy of the GNU Lesser General Public License, 
//  write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA



// **** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T ****
// ***********************************************************************************************************************************************************************
// Pinouts... IMPORTANT!!!! ... set the pinouts to your hardware configuration
//
// NEVER USE Digital pins 5,6,7,8 ... they are permanently connected to the onboard motor controller on all versions of the Rover. 
//
// Digital pins 0 and 1 can only be used if you ARE NOT using Serial communications ( ie Serial.begin() ).
// Also note that having components connected to D0 and D1 will interfere with uploading your Sketch.  
//
// 
// Pinouts on WiFi Shield explained here ... http://arduino.cc/en/Main/ArduinoWiFiShield
//
// 
// NOTE !   There is a conflict on Pin 7, used for Handshaking on the WiFi Shield, and Right Motor Direction on Rover !
// This is the workaround...
//
// Bend out Pin 7 so the WiFi Shield does not make that connection with the Rover.  Jumper 7 to 2 on the WiFi Shield. 
//
// In C:\Program Files (x86)\Arduino\libraries\WiFi\src\utility\spi_drv.cpp , change #define SLAVEREADY 2 // handshake pin
//
// FYI... using the WiFI shield with the Rover leaves you with only 4 free pins for other uses such as sensors... D3 , D9, and A2 , A3
// If you don't bother with the (basically useless) motor encoders, you'll also have A0 and A1.
//
// ***********************************************************************************************************************************************************************

// ********** Digital Pins D0 thru D13 **********

// Using 0 and 1 disables upload!     

#define DO_NOT_USE_0        0
#define DO_NOT_USE_1        1

#define DO_NOT_USE_2        2         // Pin 2 will be used for WiFi Shield handshake.  See comments above.     

#define RANGEFINDERPIN      3         // The ultrasonic rangefinder mounted on the front of the Rover (if installed).

#define DO_NOT_USE_4        4         // SS for SD Card

#define MOTORSPEEDRIGHT     5         // Nothing connected to these pins , but the onboard motor controller uses these circuits.
#define MOTORSPEEDLEFT      6
#define MOTORRIGHTDIREC     7
#define MOTORLEFTDIREC      8

#define DO_NOT_USE_10       10        // SS for WiFi
#define DO_NOT_USE_11       11        // MOSI
#define DO_NOT_USE_12       12        // MISO
#define DO_NOT_USE_13       13        // SCK


// ********** Analog Pins A0 thru A5 **********

// FYI... using analog pins for digital I/O is just the same as using digital ones.

// A0 is referred to as Pin 14     A1 is referred to as Pin 15
// A2 is referred to as Pin 16     A3 is referred to as Pin 17
// A4 is referred to as Pin 18     A5 is referred to as Pin 19

// If you don't use the encoders, you can use these pins.
#define RIGHTENCODERPIN         A0          // Right motor encoder  A0.
#define LEFTENCODERPIN          A1          // Left motor encoder   A1.


#define DO_NOT_USE_A4           A4          // SDA   Make sure you jumper SDA to A4 on the WiFi shield!
#define DO_NOT_USE_A5           A5          // SCL   Make sure you jumper SCL to A5 on the WiFi shield!



// *********************************************************************************************************************************************************************
// Required includes
// *********************************************************************************************************************************************************************


#include <SPI.h>
#include <WiFi.h>

#ifdef RANGEFINDERPIN
#include <Ultrasonic.h>
#endif


// *********************************************************************************************************************************************************************
//  Global Variables.
//  Professional programmers cringe at the thought of globals, but they seem to be generally acceptable in the "wild west" Arduino world.
//  Prudent use of them simplifies program design since variable passing via parameters between functions are reduced.
// *********************************************************************************************************************************************************************

// An homage to every program I've written since my WATFIV days.
// Every program needs an "int i", right?
int i;

char pilotCommand = 'e' , pilotCommandPrev = 'e';

char ssid[] = "RoverGate1";  // Change to your network SSID (name)  
                             // We named our Rover router after http://en.wikipedia.org/wiki/Stargate_(device)
char pass[] = "pass1234";    // Change to your network password (use for WPA, or use as key for WEP)

int keyIndex = 0;            // Your network key Index number (needed only for WEP)

int status = WL_IDLE_STATUS;

WiFiServer server(1994);
WiFiClient commChannel;

#define SERIALCOMM          // Activate this line if you want Serial Communications for debugging purposes.
                            // For example, prior to achieving full functionality, you might want to connect the Rover
                            // to a computer using the USB cable.  Then, you can use the Arduino Serial Monitor to view
                            // information messages being generated by the Rover.
                            // Once you have full functionality, you can recompile this program with this line commented out
                            // because once you are operating using WiFi only, you have no need of the Serial Communications.
                            // You won't harm anything leaving them activated, but why waste CPU cycles executing useless commands? 

  
// Motor Controls
    
#define AHEAD               LOW
#define BACK                HIGH
#define NOT                 ! 

// How fast you want the Rover to go... the stronger your battery pack, the lower your number should be.
// 100 seems to be a good setting for http://www.robotshop.com/ca/en/dfrobot-7-4v-lipo-2200mah-battery.html 

// CAUTION!   Too high a setting will draw too much power and will cause the Rover to reset when engaging both motors from a dead stop!
// If this happens, lower the speed.

#define BASELINESPEED    100
byte speedRight = BASELINESPEED, speedLeft = BASELINESPEED;


#ifdef RANGEFINDERPIN     // "ifdef's" are one of those obscure C++ features most beginning programmers don't know about.
                          // It is a neat feature that allows "conditional compilation".  In the case of this program,
                          // the various code segments using the ultrasonic sensor will only be included in the compiled executable
                          // if you defined RANGEFINDERPIN above ( ie #define RANGEFINDERPIN 3 ).
                          // For more on "conditional compilation", go to http://www.cplusplus.com/doc/tutorial/preprocessor/

// Ultrasonic variables
#define   DANGER_CLOSE    15  // Stop the Rover within this distance in cms from obstacle in front.


#endif


// **********************************************************************************************************************************************************************
// Function Prototypes                                                                                                                                                
// **********************************************************************************************************************************************************************

void roverAction          (void);                                                                // Decide on which movement to perform.
void motorSpeedDirection  (byte speedLeft, byte leftDirec, byte speedRight, byte rightDirec);    // Change speed and direction.
void motorSpeedOnly       (byte speedLeft, byte speedRight);                                     // Change speed only.

#ifdef SERIALCOMM
void WiFiStatus           (void);  // This function available only if Serial Communications are requested.
#endif

#ifdef RANGEFINDERPIN
boolean obstacleTooClose  (void);  // This function available only if ultrasonic rangefinder is included in build.
#endif

// Declare reset function @ address 0.  Calling this function is the same as pressing reset on the Arduino board.
void(* resetRover) (void) = 0; 
   
   
// *********************************************************************************************************************************************************************
// Main program                                                                                                                                                                                                                   
// *********************************************************************************************************************************************************************
  
void setup() {
  
  #ifdef SERIALCOMM
  //Initialize serial communications (ie via USB cable).
  Serial.begin(9600);
  #endif
      
  // Prepare pin for handshaking.
  pinMode(2,INPUT);
  
  // If Ultrasonic sensor installed...
  #ifdef RANGEFINDERPIN      
  pinMode(RANGEFINDERPIN,INPUT);       
  #endif

  // Check for the presence of the shield
  while ( WiFi.status() == WL_NO_SHIELD ) {  
    #ifdef SERIALCOMM  
    Serial.println("WiFi shield not present");
    #endif
    delay(1000);    
  }

  // Attempt to connect to WiFi network
  while ( status != WL_CONNECTED ) {
    
    #ifdef SERIALCOMM
    Serial.print("Attempting to connect to SSID: ");
    Serial.println(ssid);
    #endif
    
    // Connect to WPA/WPA2 network. Change this line if using Open or WEP network
    status = WiFi.begin(ssid, pass);

    // Wait 10 seconds for connection
    delay(10000);
  }

  // Start the server
  server.begin();
  
  #ifdef SERIALCOMM
  // You're connected now, report status
  WiFiStatus();
  #endif


  do {
   commChannel = server.available();
   delay(1000); 
  } while ( NOT commChannel.connected() );
  
  commChannel.flush();
  
  // http://stargate.wikia.com/wiki/Field_Remote_Expeditionary_Device
  #ifdef SERIALCOMM
  Serial.println("F.R.E.D. ready for deployment.");
  #endif
  commChannel.println("F.R.E.D. ready for deployment.");
  
  delay(1000);

}


void loop() {

  // If Ultrasonic sensor installed...
  #ifdef RANGEFINDERPIN 
  // Regardless of whether a new command is available,
  // if you're going forward and an obstacle is too close, stop immediately
  if ( pilotCommandPrev == 'w' ||  pilotCommandPrev == 'W' ) 
    if ( obstacleTooClose() ) motorSpeedOnly( 0, 0 );    
  #endif

  // If a command is in the buffer...
  if ( commChannel.available() > 0 ) {
    
    // ..read it...    
    pilotCommand = commChannel.read(); 
    
    // ...and decide what action to perform.
    roverAction();
      
    }
    
}



// *********************************************************************************************************************************************************************
// Functions <------------------------------------------------------------------------------------------------------------------------------------------------------- 
// *********************************************************************************************************************************************************************

void roverAction( void ) {

  // Incoming parameters : Nothing.
      
  // Global variables    : Uses    -   commChannel, pilotCommand
  //                       Changes -   speedLeft, speedRight
   
  // Responsibilities    : Decide what action to perform based on received pilotCommand
  
  // Returns             : Nothing.
  
  static float percModifier = 1.0;  // Used to alter motor speed
  
  switch( pilotCommand ) { // Perfom an action corresponding to pilot command received
 
          case 'w': // Move Forward
          case 'W':
  
            // If Ultrasonic sensor installed...
            #ifdef RANGEFINDERPIN    
            // If an obstacle is too close, ignore the forward request from the pilot  
            if ( obstacleTooClose() ) motorSpeedOnly( 0, 0 ); 
            else
            #endif
            
            motorSpeedDirection( speedLeft, AHEAD, speedRight, AHEAD );
            commChannel.println("Advance");
            break;
                
          case 'a': // Pivot Left at 80% current speed.  Change if desired.
          case 'A':
  
            motorSpeedDirection( speedLeft * .80, BACK, speedRight * .80, AHEAD );
            commChannel.println("Left");
            break;
                
          case 's': // Move Back
          case 'S':
    
            motorSpeedDirection( speedLeft, BACK, speedRight, BACK );
            commChannel.println("Retreat");
            break;
    
          case 'd': // Pivot Right at 80% current speed. Change if desired.
          case 'D':
  
            motorSpeedDirection( speedLeft * .80, AHEAD, speedRight * .80, BACK );
            commChannel.println("Right");
            break;
                
          case 'e': // Stop
          case 'E':
  
            motorSpeedOnly( 0, 0 ); 
            commChannel.println("Stop");
            break;

          case 'x': // Reboot
          case 'X':
          
            resetRover();
            break;  
            
          case '1': case '2': case '3': case '4':      // Modify speed ( 4 == 90% , 3 == 80% , 2 == 70% , 1 == 60% ... of BASELINESPEED )
          case '5':                                    //              ( 5 is BASELINESPEED )
          case '6': case '7': case '8': case '9':      //              ( 6 == 110% , 7 == 120% , 8 == 130% , 9 == 140% )
           
            // You may experience issues with speed control.
            // Too low and you'll hit the stall speed of the motors,
            // too high and you may draw too much current and trip the circuit protection on the Rover, causing it to reset.
            // Experimentation with BASELINESPEED may be required.
          
            // 53 is the (integer) ASCII value of digit '5'.  http://arduino.cc/en/Reference/ASCIIchart 
            
            // Using that along with the (integer) ASCII values of all the other digits 1 thru 9,
            // we can calculate values such as 1,2,3,4 which are then divided by 10.0 
            // and added/subtracted to/from 1.0 to result in values such as 1.4 (140%) , .60 (60%) , etc.
            
            // For example, the ASCII value of '9' is 57.... 57 - 53 == 4.    4/10.0 == .4   1.0 + .4 == 1.4 ... 140% when applied against BASELINESPEED
            //              the ASCII value of '1' is 49.... 53 - 49 == 4.    4/10.0 == .4   1.0 - .4 == 0.6 ...  60% 
            
            // Using "conditional assignment" ... http://www.cplusplus.com/forum/articles/14631/
            
            percModifier = 1.0                                                                           // BASELINESPEED  if pilotCommand == 5
                         + ( ( pilotCommand > '5' ) ? ( ( int(pilotCommand) - 53 ) / 10.0 ) : 0.0  )     // Increase speed if pilotCommand >  5
                         - ( ( pilotCommand < '5' ) ? ( ( 53 - int(pilotCommand) ) / 10.0 ) : 0.0  );    // Decrease speed if pilotCommand <  5
            
            // Activate the next line if you want to see the actual percentages...
            // commChannel.print("% of Baseline Speed : ");  commChannel.print(percModifier*100.0);  commChannel.println("%");  
            
            motorSpeedOnly( speedLeft = BASELINESPEED * percModifier, speedRight = BASELINESPEED * percModifier );
            
            
            break;
            
            
          case '=':  // Decrease right motor speed by 10%
          
            motorSpeedOnly( speedLeft, speedRight *= .90 );
            break;                        
            
          case '+':  // Increase right motor speed by 10%
          
            motorSpeedOnly( speedLeft, speedRight *= 1.10 );
            break;                        
            
          case '-':  // Decrease left motor speed by 10%
          
            motorSpeedOnly( speedLeft *= .90, speedRight );
            break;                        
                        
            
          case '_':  // Increase left motor speed by 10%
          
            motorSpeedOnly( speedLeft *= 1.10, speedRight );
            break;          
            
     } 
     

}

void motorSpeedDirection( byte speedLeft, byte leftDirec, byte speedRight, byte rightDirec ) {

  // Incoming parameters : byte Speed of Left Motor, byte Direction of Left Motor,
  //                       byte Speed of Right Motor, byte Direction of Right Motor.
  
  // Global variables    : Uses    -   pilotCommand
  //                       Changes -   pilotCommandPrev
  
  // Responsibilities    : Controls drive motors speed & direction.
  //                       Keep a copy of the pilotCommand just performed.
  
  // Returns             : Nothing.
  
  analogWrite(MOTORSPEEDLEFT, speedLeft);
  digitalWrite(MOTORLEFTDIREC, leftDirec);

  analogWrite(MOTORSPEEDRIGHT, speedRight);
  digitalWrite(MOTORRIGHTDIREC, rightDirec);
  
  // A direction command was performed, save its value for future use.
  pilotCommandPrev = pilotCommand;

}

void motorSpeedOnly( byte speedLeft, byte speedRight ) {

  // Incoming parameters : byte Speed of Left Motor, byte Speed of Right Motor.
    
  // Global variables    : Uses    -   commChannel
  //                       Changes -   
  
  // Responsibilities    : Controls motor speeds only.   
  
  // Returns             : Nothing.
  
  analogWrite(MOTORSPEEDLEFT, speedLeft);
  analogWrite(MOTORSPEEDRIGHT, speedRight);
  
  commChannel.print("SpeedL ");     commChannel.print(speedLeft);
  commChannel.print("  SpeedR ");   commChannel.println(speedRight);

}

#ifdef SERIALCOMM
void WiFiStatus( void ) {
  
  // Incoming parameters : Nothing.
      
  // Global variables    : Uses    -   
  //                       Changes -  
  
  // Responsibilities    : Display network info on Serial.
  
  // Returns             : Nothing.
  
  // Print the SSID of the network you're attached to
  Serial.print("SSID: ");
  Serial.println(WiFi.SSID());

  // Print your WiFi shield's IP address
  IPAddress ip = WiFi.localIP();
  Serial.print("IP Address: ");
  Serial.println(ip);

  // Print the received signal strength
  long rssi = WiFi.RSSI();
  Serial.print("signal strength (RSSI):");
  Serial.print(rssi);
  Serial.println(" dBm");
  
}
#endif

#ifdef RANGEFINDERPIN
boolean obstacleTooClose( void ) {
  
  // Incoming parameters :   Nothing.
    
  // Global variables    :   Uses    -   commChannel
  //                         Changes -   speedLeft, speedRight
  
  // Responsibilities    :   Uses Ultrasonic sensor to detect and warn about obstacle in front.
  //                         (Optionally) Will slow to 80% speed if within 2.0 x DANGER_CLOSE
  //                                      Will slow to 60% speed if within 1.5 x DANGER_CLOSE
  
  // Returns             :   true if "danger close", false otherwise.
  

  // Ultrasonic sensor object 
  static Ultrasonic ultrasonicFront(RANGEFINDERPIN);

  // This will contain the calculated distance to obstacle
  int frontRangeInCentimeters;
  
  // Used for braking
  byte speedL, speedR;
  
  ultrasonicFront.DistanceMeasure();                                                 // Get the current signal time
  frontRangeInCentimeters =   ultrasonicFront.microsecondsToCentimeters();           // Convert the time to centimeters
  
  // Possible obstacle... but because the ultrasonic is so unreliable, take a second look
  if ( frontRangeInCentimeters <= ( DANGER_CLOSE * 2 ) ) {
      
    ultrasonicFront.DistanceMeasure();                                                // Get the current signal time
    frontRangeInCentimeters += ultrasonicFront.microsecondsToCentimeters();           // Convert the time to centimeters
        
    frontRangeInCentimeters /= 2;   // Average the 2 readings
    
  }
  
  // Warn about potential threat, slow down (if desired)
  if ( frontRangeInCentimeters <= ( DANGER_CLOSE * 2 ) ) {
    
    commChannel.print("Obstacle detected at ");
    commChannel.print(frontRangeInCentimeters);
    commChannel.println(" cms.");
    
    // Closer... slower...
    // Comment out this code if you don't want this function to "apply braking".
    speedL = speedR = BASELINESPEED * ( ( frontRangeInCentimeters <= ( DANGER_CLOSE * 1.5 ) ) ? .60 : .80 );    
    motorSpeedOnly( speedL, speedR );
    
  }
  
  return ( frontRangeInCentimeters <= DANGER_CLOSE ) ? true : false;

}
#endif


